#!/bin/bash
SRC="./src"
OBJ="./obj"
BOOT="./boot"
INCNAOS="./inc"
INCGCC="/usr/local/i586-elf/include"
INCNEWLIB="/usr/local/lib/gcc/i586-elf/4.3.3/include"
NAOSOPTIONS="$NAOSOPTIONS"
GCC=naos-gcc
LD=naos-ld

export LD_LIBRARY_PATH=/usr/local/lib;

echo "Cleaning object files."
rm -f $OBJ/*

echo "Assembling kernel entry."
#nasm -f aout -o $OBJ/start.o $SRC/start.asm
nasm -f elf -o $OBJ/start.o $SRC/start.asm

# Remember this spot here: We will add 'gcc' commands here to compile C sources
echo "Compiling kernel sources."
$GCC $NAOSOPTIONS -I$INCNAOS -I$INCGCC -I$INCNEWLIB -c -o $OBJ/naos.o $SRC/naos.c
# $GCC $NAOSOPTIONS -I$INCNAOS -I$INCGCC -c -o $OBJ/naos.o $SRC/naos.c

$GCC $NAOSOPTIONS -I$INCNAOS -c -o $OBJ/screen.o $SRC/screen.c

$GCC $NAOSOPTIONS -I$INCNAOS -c -o $OBJ/gdt.o $SRC/gdt.c

$GCC $NAOSOPTIONS -I$INCNAOS -c -o $OBJ/idt.o $SRC/idt.c

$GCC $NAOSOPTIONS -I$INCNAOS -c -o $OBJ/isrs.o $SRC/isrs.c

$GCC $NAOSOPTIONS -I$INCNAOS -c -o $OBJ/irq.o $SRC/irq.c

$GCC $NAOSOPTIONS -I$INCNAOS -c -o $OBJ/timer.o $SRC/timer.c

$GCC $NAOSOPTIONS -I$INCNAOS  -I$INCGCC -I$INCNEWLIB -c -o $OBJ/kb.o $SRC/kb.c

$GCC $NAOSOPTIONS -I$INCNAOS  -I$INCGCC -I$INCNEWLIB -c -o $OBJ/cli.o $SRC/cli.c

$GCC $NAOSOPTIONS -I$INCNAOS  -I$INCGCC -I$INCNEWLIB -c -o $OBJ/uart.o $SRC/uart.c

$GCC $NAOSOPTIONS -I$INCNAOS -I$INCGCC -I$INCNEWLIB -c -o $OBJ/newlibstubs.o $SRC/newlibstubs.c

# This links all your files. Remember that as you add *.o files, you need to
# add them after start.o. If you don't add them at all, they won't be in your kernel!
echo "Linking NAOS kernel."
$LD -T link.ld \
	-o $BOOT/kernel.bin \
	$OBJ/start.o \
	$OBJ/naos.o \
	$OBJ/screen.o \
	$OBJ/gdt.o  \
	$OBJ/idt.o \
	$OBJ/isrs.o \
	$OBJ/irq.o \
	$OBJ/timer.o \
	$OBJ/kb.o \
	$OBJ/cli.o \
	$OBJ/uart.o \
	$OBJ/newlibstubs.o \
	/usr/local/i586-elf/lib/libc.a

# echo "Creating BOCHS image."

echo "Finished."
