/* File:        cli.c
 * Version:     001
 * Date:        May 31, 2009
 * Author:      hueb
 * Description: Command line interpreter test implementation.
 *
 * Change history:
 * 001 May 31, 2009 hueb Original issue 
 */

#include <stdio.h>
#include <string.h>

/* Definitions and types. */

#define NAOS_CLI_BUFSIZE 256
#define NAOS_CLI_EXIT 1

typedef int (* CliCmdFct) (void *);

typedef enum tagCliCmdName
{
	DISPLAY, 
	REBOOT
} CliCmdName;

typedef enum tagCliCmdOptionName
{
	LEN
} CliCmdOptionName;

typedef struct tagCliCmdOption
{
	CliCmdOptionName optname;
	char * optval;
} CliCmdOption;

typedef struct tagCliCmd
{
	CliCmdName cmdname;
	char * cmdstr;
	char * arg;
	CliCmdFct cmdfunction;
	unsigned char optioncount;
	CliCmdOption options;
} CliCmd;

typedef struct tagCliCmdTreeItem
{
	CliCmd * cmd;
	struct tagCliCmdTreeItem * prev;
	struct tagCliCmdTreeItem * next;
} CliCmdTreeItem;

typedef struct tagCliCmdTree
{
	CliCmdTreeItem items[26];
} CliCmdTree;

/* Functions. */

void prepareAvailableCliCommands(CliCmdTree * t)
{

}

int processCommandline(char * buf, CliCmdTree * t)
{
	if (0 == strcmp(buf, "exit\n"))
		return NAOS_CLI_EXIT;

	printf("Syntax error.\r\n");
	return 0;
}

int processCliCommands()
{
	int result = 0;
	char cmdstr[NAOS_CLI_BUFSIZE];
	CliCmdTree t;

	prepareAvailableCliCommands(&t);

	do
	{
		memset(cmdstr, 0, sizeof(cmdstr));
		printf("USER @ NAOS CLI>");
		fgets(cmdstr, NAOS_CLI_BUFSIZE, stdin);
		result = processCommandline(cmdstr, &t);
	} while (NAOS_CLI_EXIT != result);
}

int main (int argc, char * argv[])
{
	printf("NAOS command line interpreter test program\r\n");
	processCliCommands();
	return 0;
}
