/* File:        system.c
 * Version:     002
 * Date:        Jul 05, 2009
 * Author:      hueb
 * Description: NAOS system header file.
 *
 * Change history:
 * 001 Jun 12, 2009 hueb Original issue
 * 002 Jul 05, 2009 hueb UART added, formal corrections
 */

/******************************************************************************/

#ifndef __SYSTEM_H
#define __SYSTEM_H

#define NAOS_VERSION_STR "V1.0.2"
#define NAOS_STARTUP_STR "NAOS " NAOS_VERSION_STR "\r\n"

#include "cli.h"
#include "uart.h"

/* naos.c */
extern unsigned char *n_memcpy(unsigned char *dest, const unsigned char *src, int count);
extern unsigned char *n_memset(unsigned char *dest, unsigned char val, int count);
extern unsigned short *n_memsetw(unsigned short *dest, unsigned short val, int count);
extern int n_strlen(const char *str);
extern unsigned char inportb (unsigned short _port);
extern void outportb (unsigned short _port, unsigned char _data);

/* screen.c */
extern void vga_cls();
extern void vga_putc(char c);
extern int vga_puts(const char *str);
extern int vga_puti(const int value);
extern int vga_putp(const unsigned int ptr);
extern void vga_settextcolor(unsigned char forecolor, unsigned char backcolor);
extern void vga_init();

/* gdt.c */
void gdt_set_gate(int num, 
	unsigned long base, 
	unsigned long limit, 
	unsigned char access, 
	unsigned char gran);
void gdt_install();

/* idt.c */
void idt_set_gate(unsigned char num, 
	unsigned long base, 
	unsigned short sel, 
	unsigned char flags);
void idt_install();
void idt_reset();

/* isrs.c */
/* This defines what the stack looks like after an ISR was running */
struct regs
{
    unsigned int gs, fs, es, ds;      /* pushed the segs last */
    unsigned int edi, esi, ebp, esp, ebx, edx, ecx, eax;  /* pushed by 'pusha' */
    unsigned int int_no, err_code;    /* our 'push byte #' and ecodes do this */
    unsigned int eip, cs, eflags, useresp, ss;   /* pushed by the processor automatically */ 
};
void isrs_install();

/* irq.c */
void irq_install();
void irq_install_handler(int irq, void (*handler)(struct regs *r));
void irq_uninstall_handler(int irq);

/* timer.c */
void timer_install();

/* kb.c */
void keyboard_install();

/* cli.c */
void cli_reset();
void cli_putc(CliInterface * cli, unsigned char c);
void cli_reboot();
void printmem(CliInterface * cli, unsigned char * p, unsigned long len);

/* uart.c */
void uart_install();
void uart_test(unsigned short uart_port); /* For DEBUG purposes. */
void uart_puts(unsigned short uart_port, char * s);
void uart_putc(unsigned short uart_port, char c);
char uart_getc(unsigned short uart_port);

#endif /* __SYSTEM_H */
