/* File:        naos.c
 * Version:     003
 * Date:        Jul 05, 2009
 * Author:      hueb
 * Description: NAOS startup and main loop.
 *
 * Change history:
 * 001 Jan 29, 2009 hueb Original issue
 * 002 May 01, 2009 hueb Adding debug output for testing newlib.
 * 003 Jul 05, 2009 hueb UART detection and initialization added.
 */

/******************************************************************************/

/* Includes */

#include "system.h"
#include <stdio.h>
#include <malloc.h>

/******************************************************************************/

/* Globals */

/* DEBUG */
register char * stack_ptr asm ("sp");
extern char end;

/******************************************************************************/

/* Functions */

/* Function:    n_memcpy
 * Description: Barebone memcpy impementation. */
unsigned char *n_memcpy(unsigned char *dest, const unsigned char *src, int count)
{
	int i=0;
	for (i=0; i<count; i++)
		dest[i] = src[i];
	return dest;
}

/* Function:    n_memset
 * Description: Barebone memset impementation. */
unsigned char *n_memset(unsigned char *dest, unsigned char val, int count)
{
	int i=0;
	for (i=0; i<count; i++)
		dest[i] = val;
	return dest;
}

/* Function:    n_memsetw
 * Description: Barebone memsetw impementation. */
unsigned short *n_memsetw(unsigned short *dest, unsigned short val, int count)
{
    /* Same as above, but this time, we're working with a 16-bit
     * 'val' and dest pointer. Your code can be an exact copy of
     * the above, provided that your local variables if any, are
     * unsigned short */
	int i=0;
	for (i=0; i<count; i++)
		dest[i] = val;
	return dest;
}

/* Function:    n_strlen
 * Description: Barebone n_strlen impementation. */
int n_strlen(const char *str)
{
	int i = 0;
	for (i=0; 0!=str[i]; i++) ;
	return i;
}


/* Function:    inportb
 * Description: Reads from a processor port.
 *              We will use this for reading from the I/O ports to get data
 *              from devices such as the keyboard. We are using what is called
 *              'inline assembly' in these routines to actually do the work */
unsigned char inportb (unsigned short _port)
{
    unsigned char rv;
    __asm__ __volatile__ ("inb %1, %0" : "=a" (rv) : "dN" (_port));
    return rv;
}

/* Function:    inportb
 * Description: Writes to a processor port.
 *              We will use this to write to I/O ports to send bytes to 
 *              devices. This will be used in the next tutorial for
 *              changing the textmode cursor position. Again, we use some
 *              inline assembly for the stuff that simply 
 *              cannot be done in C */
void outportb (unsigned short _port, unsigned char _data)
{
    __asm__ __volatile__ ("outb %1, %0" : : "dN" (_port), "a" (_data));
}

/* Function:    main
 * Description: NAOS startup and main loop. */
int main()
{
	/* Init hardware. */
	gdt_install();
	idt_install();
	isrs_install();
	irq_install();
	vga_init();
	timer_install();
	keyboard_install();

	vga_puts(NAOS_STARTUP_STR);

	uart_install();
	
	cli_init();
	vga_puts(CLI_STARTUP_STR);

	/* Open topic: set stack ptr dynamically. */
/*	puts("Stack Pointer: ");
	putp((const unsigned int) stack_ptr);
	puts("\r\nEnd: ");
	putp((const unsigned int) &end); */

	/* Open topic: newlib testing. */
/*	printf("\r\nHello newlib\r\n");

	{
		int i = 0;
		unsigned char * p[i+10];
		n_memset ((void*)p, 0, sizeof(p));

		for (i=0; i<10; i++)
			p[i+i] = (unsigned char *) malloc(0x10*(i+1));

		for (i=0; i<10; i++)
			printf("p[i+%d] = 0x%08X, dp = 0x%08X\r\n", i, (unsigned)p[i+i], i ? (unsigned)(p[i+i]-p[i+i-1]) : 0);

		printmem(p[i+0], 40);

		for (i=0; i<10; i++)
			free(p[i+i]);
	}
*/

	/* Test Exceptions */
	/* i = 10 / 0;
	putch(i);
	puts("This may not be printed\r\n"); */

	/* Activate IRQs */
	__asm__ __volatile__ ("sti");
	
	/* asm volatile ("int $0x20"); */

	for (;;);

	return 0;
}
