/* File:        newlibstubs.c
 * Version:     001
 * Date:        Apr 09, 2009
 * Author:      hueb
 * Description: Glue layer for integration of newlib into NAOS.
 *
 * Change history:
 * 001 Apr 09, 2009 hueb Original issue
 */

/******************************************************************************/

/* Includes */

#include <system.h>
#include <sys/times.h>
#include <errno.h>

/******************************************************************************/

/* Defintions */

#undef errno

/******************************************************************************/

/* Globals */

extern int errno;

/* SP definition follows arm architecture. See libgloss/arm/syscalls.c */
register char * stack_ptr asm ("esp");

/******************************************************************************/

/* Functions */

/* Close a file. */
int close(int file)
{
	return -1;
}

/* Environment. */
char *__env[1] = { 0 };
char **environ = __env;

/* Transfer control to a new process.
 * Minimal implementation for systems without processes. */
int execve(char *name, char **argv, char **env)
{
	errno = ENOMEM;
	return -1;
}

/* Create a new process.
 * Minimal implementation for systems without processes. */
int fork(void)
{
	errno = EAGAIN;
	return -1;
}

/* Status of an open file. */
#include <sys/stat.h>
int fstat(int file, struct stat *st)
{
	st->st_mode = S_IFCHR;
	return 0;
}

/* Process ID.
 * Minimal implementation for systems without processes. */
int getpid(void)
{
	return 1;
}

/* Query whether output stream is a terminal.
 * Only stdout. */
int isatty(int file)
{
	return 1;
}

/* Send a signal. */
int kill(int pid, int sig)
{
	errno = EINVAL;
	return -1;
}

/* Establish a new name for an existing file. */
int link(char *old, char *new)
{
	errno = EMLINK;
	return -1;
}

/* Set position in a file. */
int lseek(int file, int ptr, int dir)
{
	return 0;
}

/* Open a file. */
int open(const char *name, int flags, int mode)
{
	return -1;
}

/* Read from a file. */
int read(int file, char *ptr, int len)
{
	return 0;
}

void abort(void)
{
	/* Raise exception */
	puts("SIGABRT\r\n");
	asm volatile ("int $0x04");
}

/* Write to a file. */
int write(int file, char *ptr, int len)
{
/*	int todo;
	
	for (todo = 0; todo < len; todo++)
	{
		outbyte (*ptr++);
	}*/

	puts(ptr);

	return len;
}

/* Increase program data space.
 * Malloc backend. */
caddr_t sbrk(int incr)
{
	extern char end;		/* Defined by the linker */
	static char *heap_end;
	char *prev_heap_end;

	if (heap_end == 0)
	{
		heap_end = &end;
	}

	prev_heap_end = heap_end;

	if (heap_end + incr > stack_ptr)
	{
		write (1, "Heap and stack collision\n", 25);
		abort ();
	}

	heap_end += incr;
	return (caddr_t) prev_heap_end;
}

/* Status of a file (by name). */
int stat(const char *file, struct stat *st)
{
	st->st_mode = S_IFCHR;
	return 0;
}

/* Timing information for current process. */
clock_t times(struct tms *buf)
{
	return (clock_t)-1;
}

/* Remove a file's directory entry. */
int unlink(char *name)
{
	errno = ENOENT;
	return -1;
}

/* Wait for a child process. */
int wait(int *status)
{
	errno = ECHILD;
	return -1;
}
