/* File:        timer.c
 * Version:     001
 * Date:        Jan 18, 2009
 * Author:      hueb
 * Description: Timer handling.
 *
 * Change history:
 * 001 Jan 18, 2009 hueb Original issue
 */

/******************************************************************************/

/* Includes */

#include "system.h"

/******************************************************************************/

/* Globals */

/* This will keep track of how many ticks that the system
 * has been running for */
int timer_ticks = 0;

/******************************************************************************/

/* Functions */

/* Function:    timer_phase
 * Description: Computes the control value to write to the timer port according
 *              to a given frequency in [HZ]. */
void timer_phase(int hz)
{
    int divisor = 1193180 / hz;       /* Calculate our divisor */
    outportb(0x43, 0x36);             /* Set our command byte 0x36 */
    outportb(0x40, divisor & 0xFF);   /* Set low byte of divisor */
    outportb(0x40, (divisor >> 8) & 0xFF);     /* Set high byte of divisor */
}

/* Function:    timer_handler
 * Description: Handles the timer. In this case, it's very simple: We
 *              increment the 'timer_ticks' variable every time the
 *              timer fires. By default, the timer fires 18.222 times
 *              per second. Why 18.222Hz? Some engineer at IBM must've
 *              been smoking something funky. */
void timer_handler(struct regs *r)
{
    /* Increment our 'tick count' */
    timer_ticks++;

    /* Every 18 clocks (approximately 1 second), we will
     * display a message on the screen */
    if (timer_ticks % 100 == 0)
    {
        // puts("One second has passed\r\n");
    }
}

/* Function:    timer_install
 * Description: Sets up the system clock by installing the timer handler
 *              into IRQ0 */
void timer_install()
{
    /* Installs 'timer_handler' to IRQ0 */
    irq_install_handler(0, timer_handler);
	timer_phase(100);
}
